//
//  BMKMultiRouteSelectedView.m
//  BMKMultiRoutePlanDemo
//
//  Created by zhangbaojin on 2020/5/22.
//  Copyright © 2020 zhangbaojin. All rights reserved.
//


#define kDefaultColor [UIColor blackColor]
#define kHeighLightColor [UIColor blueColor]

#import "BMKMultiRouteSelectedView.h"
#import "BNCarRouteData.h"

@interface BMKRouteBoxView ()

/// name
@property (nonatomic, strong) UILabel *nameLabel;

/// time
@property (nonatomic, strong) UILabel *timeLabel;

/// distance
@property (nonatomic, strong) UILabel *distanceLabel;

/// block
@property (nonatomic, copy) void(^selectedBlock)(NSUInteger index);

@end

@implementation BMKRouteBoxView

- (instancetype)init {
    if (self = [super init]) {
        _nameLabel = [UILabel new];
        _nameLabel.textAlignment = NSTextAlignmentCenter;
        _nameLabel.font = [UIFont systemFontOfSize:15];
        _nameLabel.textColor = kDefaultColor;
        [self addSubview:_nameLabel];
        
        _timeLabel = [UILabel new];
        _timeLabel.textAlignment = NSTextAlignmentCenter;
        _timeLabel.font = [UIFont systemFontOfSize:19];
        _timeLabel.textColor = kDefaultColor;
        [self addSubview:_timeLabel];
        
        _distanceLabel = [UILabel new];
        _distanceLabel.textAlignment = NSTextAlignmentCenter;
        _distanceLabel.font = [UIFont systemFontOfSize:15];
        _distanceLabel.textColor = kDefaultColor;
        [self addSubview:_distanceLabel];
        
        UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(boxTap:)];

        [self addGestureRecognizer:tapGesture];
    }
    return self;
}

- (void)boxTap:(UITapGestureRecognizer *)gesture {
    [self highlighted];
    if (self.selectedBlock) {
        self.selectedBlock(self.tag);
    }
}

- (void)selected:(void (^)(NSUInteger index))selectedBlock {
    self.selectedBlock = selectedBlock;
}
- (void)layoutSubviews {
    CGFloat width = self.bounds.size.width;
    _nameLabel.frame = CGRectMake(0, 18, width, 25);
    _timeLabel.frame = CGRectMake(0, CGRectGetMaxY(_nameLabel.frame), width, 30);
    _distanceLabel.frame = CGRectMake(0, CGRectGetMaxY(_timeLabel.frame), width, 25);
}

- (void)configInfo:(BNCarOneRouteModel *)info{
    _nameLabel.text = info.routeLabelName;
    _timeLabel.text = [NSString stringWithFormat:@"%.1lf小时", (info.time / (60.0 * 60))];
    _distanceLabel.text = [NSString stringWithFormat:@"%.1lf公里", (info.distance / 1000.0)];
}
- (void)restore {
    _nameLabel.textColor = kDefaultColor;
    _timeLabel.textColor = kDefaultColor;
    _distanceLabel.textColor = kDefaultColor;
}
- (void)highlighted {
    _nameLabel.textColor = kHeighLightColor;
    _timeLabel.textColor = kHeighLightColor;
    _distanceLabel.textColor = kHeighLightColor;
}
@end

@interface BMKMultiRouteSelectedView ()

/// 多路线，NSMutableArray的内容为
@property (nonatomic, copy) NSArray<BNCarOneRouteModel *> *carRoutes;

/// 选中回调
@property (nonatomic, copy) void(^selectedBlock)(NSUInteger index);
@end

@implementation BMKMultiRouteSelectedView

- (instancetype)init {
    if ([super init]) {
        self.backgroundColor = [UIColor whiteColor];
    }
    return self;
}
// 创建子视图
- (void)initSubViews {
    CGFloat width = [[UIScreen mainScreen] bounds].size.width / self.carRoutes.count;
        CGFloat height = 140;
        for (int i = 0; i < self.carRoutes.count; i++) {
          __block  BMKRouteBoxView *box = [BMKRouteBoxView new];
            box.tag = i;
            box.frame = CGRectMake(i * width, 0, width, height);
            [box configInfo:self.carRoutes[i]];
            [self addSubview:box];
            __weak typeof(self) weakSelf = self;
            [box selected:^(NSUInteger index) {
                [box highlighted];
                [weakSelf boxTap:index];
            }];
        }
}
// box点击事件
- (void)boxTap:(NSUInteger)index {
    if (self.selectedBlock) {
        self.selectedBlock(index);
    }
    // 其它恢复初始状态
    [self.subviews enumerateObjectsUsingBlock:^(__kindof UIView * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        if ([obj isKindOfClass:[BMKRouteBoxView class]]) {
            
            if (obj.tag != index) {
                [(BMKRouteBoxView *)obj restore];
            }
        }
    }];
    NSLog(@"选中了第%d条路线", (int)index);
    
}
/// 配置路线数据并生成对应的子视图
- (void)configData:(NSArray<BNCarOneRouteModel *> *)routes{
    if (!routes) return;
    self.carRoutes = [routes copy];
    [self initSubViews];
}
/// 设置选中第几条路线
- (void)multiRouteSelected:(NSUInteger)index {
    [self.subviews enumerateObjectsUsingBlock:^(__kindof UIView * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
        if ([obj isKindOfClass:[BMKRouteBoxView class]]) {
            
            if (obj.tag == index) {
                [(BMKRouteBoxView *)obj highlighted];
            } else {
                 [(BMKRouteBoxView *)obj restore];
            }
        }
    }];
}
/// 方案选中回调
- (void)routeSelected:(void(^)(NSUInteger index))selectedBlock{
    self.selectedBlock = selectedBlock;
}
@end




