//
//  BMKMultiRouteViewController.m
//  BMKMultiRoutePlanDemo
//
//  Created by zhangbaojin on 2020/5/21.
//  Copyright © 2020 zhangbaojin. All rights reserved.
//


#import "BMKMultiRouteViewController.h"
#import "BNCarRouteData.h"
#import "BNaviService.h"

#import "BMKMultiRouteSelectedView.h"
@interface BMKMultiRouteViewController ()<BNNaviRoutePlanDelegate,  BNDriveRouteManagerDelegate, BNMapViewManagerDelegate, BNNaviUIManagerDelegate>

/// 算路策略
@property (nonatomic, assign) BNRoutePlanMode strategy;
/// 多路线总信息
@property (nonatomic, strong) BNCarRouteModel *carRoute;

/// 算路点
@property (nonatomic, strong) NSArray<BNRoutePlanNode *> *nodes;

/// 标记选中路线的index
@property (nonatomic, assign) NSInteger selectIndex;

/// 选择框
@property (nonatomic, strong) BMKMultiRouteSelectedView *multiRouteView;

/// 导航btn
@property (nonatomic, strong) UIButton *naviBtn;
@end

@implementation BMKMultiRouteViewController

- (void)dealloc {
    [BNaviService_DriveRoute stopUpdateLocation];
    NSLog(@"DrivePageViewController Dealloc");
}

- (NSString *)title {
    return @"多路线规划";
}
- (NSArray<BNRoutePlanNode *> *)nodes{
    if (!_nodes) {
        BNRoutePlanNode *start = [BNRoutePlanNode new];
        BNPosition *sP = [BNPosition new];
        
//        sP.x = 121.44547;
//        sP.y = 31.264084;
        sP.x = 116.307432;
        sP.y = 40.056884;
        sP.eType = BNCoordinate_BaiduMapSDK;
        start.pos = sP;

        BNRoutePlanNode *end = [BNRoutePlanNode new];
        BNPosition *eP = [BNPosition new];
//        eP.x = 117.193047;
//        eP.y = 39.159697;
        eP.x = 116.328103;
        eP.y = 39.900835;
        eP.eType = BNCoordinate_BaiduMapSDK;
        end.pos = eP;
        _nodes = @[start, end];
    }
    return _nodes;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor whiteColor];
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView:self.naviBtn];
    self.strategy = BNRoutePlanMode_Recommend;
    BNaviService_DriveRoute.delegate = self;
    BNaviService_MapView.delegate = self;
    
    // 算路
    [self startRoutePlan];
    // 创建地图
    [self initMapViewStatus];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    self.navigationController.interactivePopGestureRecognizer.enabled = NO;
    [BNaviService_DriveRoute viewWillAppear:self.view];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    
    self.navigationController.interactivePopGestureRecognizer.enabled = YES;
    [BNaviService_DriveRoute viewWillDisAppear:self.view];
}

#pragma mark - UI
- (UIButton *)naviBtn {
    if (!_naviBtn) {
        _naviBtn = [UIButton new];
//        [_naviBtn setTitle:@"导航" forState:UIControlStateNormal];
        [_naviBtn setImage:[UIImage imageNamed:@"icon_navi"] forState:UIControlStateNormal];
        [_naviBtn addTarget:self action:@selector(naviBtnClicked:) forControlEvents:UIControlEventTouchUpInside];
        [_naviBtn sizeToFit];
    }
    return _naviBtn;
}

- (void)initMapSubView {
    self.multiRouteView = [BMKMultiRouteSelectedView new];
    self.multiRouteView.frame = CGRectMake(0, CGRectGetHeight(self.view.frame) - 140, KScreenWidth, 140);
    [self.view addSubview:self.multiRouteView];
}

// 初始化底图状态
- (void)initMapViewStatus {
    BNMapViewStatus* mapViewStatus = [BNaviService_MapView getMapViewStatus];
    mapViewStatus.level = 15;
    if (self.nodes.count) {
        BNRoutePlanNode *startNode = self.nodes.firstObject;
        mapViewStatus.center = startNode.pos;
    }
    
    [BNaviService_MapView setMapViewStatus:mapViewStatus animated:NO];
}
#pragma mark - Private
- (void)routeSelectIndex:(NSInteger)selectIndex needSelectRoute:(BOOL)needSelectRoute {
    self.selectIndex = selectIndex;
    if (needSelectRoute) {
        [BNaviService_DriveRoute selectRouteAtIndex:selectIndex];
    }
}

- (CGFloat)bottomPaneHeight {
    if (self.carRoute.carRoutes.count == 1) {
        return kPanelLessHeight;
    } else {
        return kPanelHeight;
    }
}
// 开始导航
- (void)naviBtnClicked:(UIButton *)btn {
    [BNaviService_UI showPage:BNaviUI_NormalNavi delegate:self extParams:nil];
}
/// 发起算路
- (void)startRoutePlan {
    if (self.nodes.count >= 2) {
        BNaviService_RoutePlan.disableOpenUrl = YES;
        [BNaviService_RoutePlan startNaviRoutePlan:BNRoutePlanMode_Recommend naviNodes:self.nodes time:nil delegete:self userInfo:nil];
    }
}
/// 屏幕适配
- (void)showRouteViewAll {
    BNMargin margin = {80, 10, CGRectGetHeight(self.view.frame) - (KScreenHeight-[self bottomPaneHeight]-(kRAButtonHeight*2+1)- 15), 20};
    [BNaviService_DriveRoute showRouteViewAll:margin animated:YES];
}
#pragma mark - BNNaviRoutePlanDelegate

/// 算路成功回调
/// @param userInfo 用户信息
- (void)routePlanDidFinished:(NSDictionary *)userInfo {
    NSLog(@"算路成功回调 = %@", userInfo);
    [self showRouteViewAll];
    [self initMapSubView];
    BNCarRouteModel *route = userInfo[BNDriveRouteDataKey];
    if (route) {
        self.carRoute = route;
    }
    // 配置路线数据
    [self.multiRouteView configData:self.carRoute.carRoutes];
    // 默认选中第一条
    [self.multiRouteView multiRouteSelected:0];
    
    // 路线选中回调
    __weak typeof(self) weakSelf = self;
    [self.multiRouteView routeSelected:^(NSUInteger index) {
        [weakSelf routeSelectIndex:index needSelectRoute:YES];
    }];
}

/// 算路失败回调
/// @param error 失败信息
/// @param userInfo 用户信息
- (void)routePlanDidFailedWithError:(NSError *)error andUserInfo:(NSDictionary *)userInfo {
    NSLog(@"算路失败回调 error = %@ \n userInfo= %@", [error description], userInfo);

}

///  算路取消
/// @param userInfo 用户信息
-(void)routePlanDidUserCanceled:(NSDictionary *)userInfo {
    NSLog(@"算路取消 userInfo = %@", userInfo);
}
#pragma mark - BNDriveRouteManagerDelegate

/// 用户在地图上点击了某条路线（如果需要高亮该路线，要调用selectRouteAtIndex:接口）
/// @param routeIndex 路线序号（从0开始）
- (void)onHandleTouchRouteAtIndex:(NSUInteger)routeIndex {
    NSLog(@"用户在地图上点击了第 %lu 条路线", (unsigned long)routeIndex);
    if (self.multiRouteView) {
        [self.multiRouteView multiRouteSelected:routeIndex];
    }
}

/// 回调车点
/// @param positionInfo 车点信息
- (void)onHandleUpdateCurrentCarPositionInfo:(NSDictionary *)positionInfo {
    
}
#pragma mark - BNMapViewManagerDelegate

/// 地图缩放级别发生变化回调
/// @param level 缩放级别
- (void)onHandleMapLevelChange:(NSInteger)level {

}

#pragma mark - BNNaviUIManagerDelegate
/**
 *  导航页面的调起controller
 *  可不实现，默认为最上层的controller
 */
- (id)naviPresentedViewController {
    return self;
}

/**
 *  即将退出UI的回调
 *
 *  @param pageType  UI类型
 *  @param extraInfo 额外参数
 */
- (void)willExitPage:(BNaviUIType)pageType  extraInfo:(NSDictionary *)extraInfo {
    
}

/**
 *  退出UI的回调
 *
 *  @param pageType  UI类型
 *  @param extraInfo 额外参数
 */
- (void)onExitPage:(BNaviUIType)pageType  extraInfo:(NSDictionary *)extraInfo {
    [self showRouteViewAll];
}
#pragma mark - 禁止横屏
- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskPortrait;
}

- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation {
    return UIInterfaceOrientationPortrait;
}





@end
